# -*- Mode: Python; tab-width: 4 -*-

import asynchat
import socket
import string

class ident_client (asynchat.async_chat):

	def __init__ (self, (remote_port,  local_port), remote_ip, callback):
		self.port_pair = (remote_port, local_port)
		self.remote_ip = remote_ip
		self.callback = callback
		self.set_terminator ('\r\n')
		self.data = ''
		asynchat.async_chat.__init__ (self)
		self.create_socket (socket.AF_INET, socket.SOCK_STREAM)
		self.connect ((remote_ip, 113))

	def handle_connect (self):
		self.send ('%d, %d\r\n' % self.port_pair)
		
	def handle_error (self, *info):
		# ignore all problems with ident
		self.close()

	def collect_incoming_data (self, data):
		if len(self.data) > 1000:
			# watch for naughty implementations...
			self.close()
		else:
			self.data = self.data + data

	def found_terminator (self):
		# sample replies:
		# 6193, 23 : USERID : UNIX : stjohns
		# 6195, 23 : ERROR : NO-USER
		line, self.data = self.data, ''
		parts = string.split (line, ':')
		if len(parts) >= 2:
			response = string.lower (string.strip (parts[1]))
			if response == 'userid':
				self.callback (string.strip (parts[3]))
		self.close()




# -*- Mode: Python; tab-width: 4 -*-

class multi_filter:
	"""Combines multiple filters into a single filter object.
	The response is the logical OR of the sub-filters"""

	def __init__ (self, *filters):
		self.filters = filters
		self.hits = 0

	def scan (self, headers):
		for f in self.filters:
			if f.scan (headers):
				self.hits = self.hits + 1
				return 1
		return 0

	def report (self, channel):
		channel.push ('multi-filter report (%d hits total):\r\n' % self.hits)
		for f in self.filters:
			channel.push (' --- filter %s:\r\n' % repr(f))
			f.report (channel)
		channel.push ('done with report.\r\n')



# -*- Mode: Python; tab-width: 4 -*-

import regex

class regex_filter:
	"""Simple filter class.
	The `filters' argument is a list of filters.  Each filter
	is a list of regular expressions that will be matched against
	headers in each message"""

	def __init__ (self, filters, do_promote=1):

		self.hit_dict = {}
		for filter in filters:
			self.hit_dict[id(filter)] = 0

		self.filters = filters
		self.do_promote = do_promote

	def scan (self, headers):
		"Return true if there's a match"
		i = 0
		for filter in self.filters:
			if self.try_filter (headers, filter):
				fid = id(filter)
				self.hit_dict[fid] = self.hit_dict[fid] + 1
				if self.do_promote:
					self.promote (i)
				return 1
			i = i + 1
		return 0

	def try_filter (self, headers, f):
		# a multi-regex filter is a logical AND of several regex matches
		# we short-circuit the AND by returning false when one of the regex
		# fails to match.
		for r in f:
			# if all are -1, then this regex fails. return false.
			if not len (filter (lambda x: x!=-1, map (r.search, headers))):
				# any failures, then we don't match
				return 0
		# made it through, we have a hit!
		return 1

	def promote (self, index):
		"move filter #index to the top"
		self.filters.insert (0, self.filters[index])
		del self.filters[index+1]

	def report (self, channel):
		# report on the top 10 hits
		for i in range(10):
			f = self.filters[i]
			channel.push (
				' <%s>: %d hits\r\n' % (
					repr (map (lambda x: x.givenpat, f)),
					self.hit_dict[id(f)]
					)
				)



# -*- Mode: Python; tab-width: 4 -*-

# An automatically-generated list of spammers
# (compiled via spam-list@spam-archive.org)
# is available from:
#   ftp://ftp.hiss.org/pub/sendmail/filter.cf.data

import regex
import rfc822
import string

def strip_eol (line):
	while line and line[-1] in "\r\n":
		line = line[:-1]
	return line

class blacklist_filter:
	def __init__ (self, filename):
		d = {}
		f = open (filename, 'r')
		print 'reading blacklist...'
		while 1:
			line = f.readline()
			if not line:
				break
			elif line[0] == '#':
				continue
			else:
				line = strip_eol (line)
				d[line] = 0
		print 'done.'
		self.bad_guys = d

	def scan (self, headers):
		for h in headers:
			if h[:6] == 'From: ':
				name, addr = rfc822.parseaddr (h[6:])
				if self.bad_guys.has_key (addr):
					self.bad_guys[addr] = self.bad_guys[addr] + 1
					return 1
		return 0

	def report (self, channel):
		list = map (lambda x: (x[1],x[0]), self.bad_guys.items())
		list.sort()
		ll = len(list)
		for i in range(10):
			elem = list[ll-i-1]
			print elem
			channel.push (
				' <%s>: %d hits\r\n' % (
					elem[1],
					elem[0]
					)
				)



# -*- Mode: Python; tab-width: 4 -*-
# 	$Id: asyncore.py,v 2.12 1997/08/25 16:36:32 rushing Exp $
#	Author: Sam Rushing <rushing@nightmare.com>

# ======================================================================
# Copyright 1996 by Sam Rushing
# 
#                         All Rights Reserved
# 
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby
# granted, provided that the above copyright notice appear in all
# copies and that both that copyright notice and this permission
# notice appear in supporting documentation, and that the name of Sam
# Rushing not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission.
# 
# SAM RUSHING DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN
# NO EVENT SHALL SAM RUSHING BE LIABLE FOR ANY SPECIAL, INDIRECT OR
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
# OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
# NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
# CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
# ======================================================================

import select
import socket
import string
import sys

import os
if os.name == 'nt':
	EWOULDBLOCK	= 10035
	EINPROGRESS	= 10036
	EALREADY	= 10037
	ECONNRESET  = 10054
	ENOTCONN	= 10057
else:
	from errno import EALREADY, EINPROGRESS, EWOULDBLOCK, ECONNRESET, ENOTCONN

socket_map = {}

def poll (timeout=0.0):
	if socket_map:
		sockets = socket_map.keys()
		r = filter (lambda x: x.readable(), sockets)
		w = filter (lambda x: x.writable(), sockets)
		e = sockets[:]

		(r,w,e) = select.select (r,w,e, timeout)

		for x in e:
			try:
				x.handle_expt_event()
			except:
				x.handle_error (sys.exc_type, sys.exc_value, sys.exc_traceback)
		for x in r:
			try:
				x.handle_read_event()
			except:
				x.handle_error (sys.exc_type, sys.exc_value, sys.exc_traceback)
		for x in w:
			try:
				x.handle_write_event()
			except:
				x.handle_error (sys.exc_type, sys.exc_value, sys.exc_traceback)


def loop (timeout=30.0):
	while socket_map:
		poll (timeout)

class dispatcher:
	debug = 0
	connected = 0
	accepting = 0
	closing = 0
	_fileno = None
	addr = None

	def __init__ (self, sock=None):
		if sock:
			self.set_socket (sock)
			# I think it should inherit this anyway
			self.socket.setblocking (0)
			self.connected = 1

	def __repr__ (self):
		status = []
		if self.accepting and self.addr:
			status.append ('listening')
		elif self.connected:
			status.append ('connected')
		if self.addr:
			status.append ('%s:%d' % self.addr)
		return '<%s %s at %x>' % (
			self.__class__.__name__,
			string.join (status, ' '),
			id(self)
			)

	def add_channel (self):
		self.log ('adding channel %s' % self)
		socket_map [self] = 1

	# we cache the original fileno, because after closing
	# a socket, s.fileno() will return -1, and we want to
	# continue tracking it via the original number.
	def fileno (self):
		if self._fileno is None:
			self._fileno = self.socket.fileno()
		return self._fileno

	def del_channel (self):
		if socket_map.has_key (self):
			self.log ('closing channel %d:%s' % (self.fileno(), self))
			del socket_map [self]

	def create_socket (self, family, type):
		self.socket = socket.socket (family, type)
		self.socket.setblocking(0)
		self.add_channel()

	def set_socket (self, socket):
		self.socket = socket
		self.add_channel()

	# ==================================================
	# predicates for select()
	# these are used as filters for the lists of sockets
	# to pass to select().
	# ==================================================
	def readable (self):
		return 1

	def writable (self):
		return 1

	# ==================================================
	# socket object methods.
	# ==================================================

	def listen (self, num):
		self.accepting = 1
		if os.name == 'nt' and num > 5:
			num = 1
		return self.socket.listen (num)

	def bind (self, addr):
		self.addr = addr
		return self.socket.bind (addr)

	def connect (self, address):
		try:
			self.socket.connect (address)
		except socket.error, why:
			if why[0] in (EINPROGRESS, EALREADY, EWOULDBLOCK):
				return
			else:
				raise socket.error, why
		self.connected = 1
		self.handle_connect()

	def accept (self):
		try:
			conn, addr = self.socket.accept()
			return conn, addr
		except socket.error, why:
			if why[0] == EWOULDBLOCK:
				pass
			else:
				raise socket.error, why

	def send (self, data):
		try:
			result = self.socket.send (data)
			return result
		except socket.error, why:
			if why[0] == EWOULDBLOCK:
				return 0
			else:
				raise socket.error, why
			return 0

	def recv (self, buffer_size):
		try:
			data = self.socket.recv (buffer_size)
			if not data:
				# a closed connection is indicated by signaling
				# a read condition, and having recv() return 0.
				self.handle_close()
				return ''
			else:
				return data
		except socket.error, why:
			# winsock sometimes throws ENOTCONN
			if why[0] in [ECONNRESET, ENOTCONN]:
				self.handle_close()
				return ''
			else:
				raise socket.error, why

	def close (self):
		self.socket.close()
		self.del_channel()

	# cheap inheritance, used to pass all other attribute
	# references to the underlying socket object.
	def __getattr__ (self, attr):
		if attr != 'socket':
			return getattr (self.socket, attr)
		else:
			raise AttributeError, attr

	def log (self, message):
		print 'log:', message

	def handle_read_event (self):
		if self.accepting:
			# for an accepting socket, getting a read implies
			# that we are connected
			if not self.connected:
				self.connected = 1
			self.handle_accept()
		elif not self.connected:
			self.handle_connect()
			self.connected = 1
			self.handle_read()
		else:
			self.handle_read()

	def handle_write_event (self):
		# getting a write implies that we are connected
		if not self.connected:
			self.handle_connect()
			self.connected = 1
		self.handle_write()

	def handle_expt_event (self):
		self.handle_expt()

	def handle_error (self, *info):
		(t,v,etb) = info
		import tb
		tb.printtb (etb)
		while etb.tb_next:
			etb = etb.tb_next
		print (
			'uncaptured python exception, closing channel %s (%s:%s file:%s line:%d)' % (
				self,
				str(t),
				str(v),
				etb.tb_frame.f_code.co_filename,
				etb.tb_lineno
				)
			)
		self.close()

	def handle_expt (self):
		self.log ('unhandled exception')

	def handle_read (self):
		self.log ('unhandled read event')

	def handle_write (self):
		self.log ('unhandled write event')

	def handle_connect (self):
		self.log ('unhandled connect event')

	def handle_oob (self):
		self.log ('unhandled out-of-band event')

	def handle_accept (self):
		self.log ('unhandled accept event')

	def handle_close (self):
		self.log ('unhandled close event')
		self.close()

# ---------------------------------------------------------------------------
# adds simple buffered output capability, useful for simple clients.
# [for more sophisticated usage use asynchat.async_chat]
# ---------------------------------------------------------------------------

class dispatcher_with_send (dispatcher):
	def __init__ (self, sock=None):
		dispatcher.__init__ (self, sock)
		self.out_buffer = ''

	def initiate_send (self):
		num_sent = 0
		num_sent = dispatcher.send (self, self.out_buffer[:512])
		self.out_buffer = self.out_buffer[num_sent:]

	def handle_write (self):
		self.initiate_send()

	def writable (self):
		return (not self.connected) or len(self.out_buffer)

	def send (self, data):
		if self.debug:
			self.log ('sending %s' % repr(data))
		self.out_buffer = self.out_buffer + data
		self.initiate_send()

# ---------------------------------------------------------------------------
# used for debugging.
# ---------------------------------------------------------------------------

def close_all ():
	for x in socket_map.keys():
		x.socket.close()
	socket_map = {}

# Asynchronous File I/O:
#
# After a little research (reading man pages on various unixen, and
# digging through the linux kernel), I've determined that select()
# isn't meant for doing doing asynchronous file i/o.
# Heartening, though - reading linux/mm/filemap.c shows that linux
# supports asynchronous read-ahead.  So _MOST_ of the time, the data
# will be sitting in memory for us already when we go to read it.
#
# What other OS's (besides NT) support async file i/o?  [VMS?]
#
# Regardless, this is useful for pipes, and stdin/stdout...

import os
if os.name == 'posix':
	import fcntl
	import FCNTL

	class file_wrapper:
		# here we override just enough to make a file
		# look like a socket for the purposes of asyncore.
		def __init__ (self, fd):
			self.fd = fd

		def recv (self, *args):
			return apply (os.read, (self.fd,)+args)

		def write (self, *args):
			return apply (os.write, (self.fd,)+args)

		def close (self):
			return os.close (self.fd)

		def fileno (self):
			return self.fd

	class file_dispatcher (dispatcher):
		def __init__ (self, fd):
			dispatcher.__init__ (self)
			self.connected = 1
			# set it to non-blocking mode
			flags = fcntl.fcntl (fd, FCNTL.F_GETFL, 0)
			flags = flags | FCNTL.O_NONBLOCK
			fcntl.fcntl (fd, FCNTL.F_SETFL, flags)
			self.set_file (fd)

		def set_file (self, fd):
			self.socket = file_wrapper (fd)
			self.add_channel()



# -*- Mode: Python; tab-width: 4 -*-
#
# A good list of regexps (in procmail-ready form) is available from
#   http://alcor.concordia.ca/topics/email/auto/procmail/spam/tag.html
#

# I know nothing about procmail, and remember very little about perl,
# so this parser is made with simple eyeball knowledge.  This script
# will simply extract the regexps, ignoring all the perl/procmail/whatnot.

import regex
import regex_syntax
import string
import sys

def strip_eol (line):
	while line and line[-1] in "\r\n":
		line = line[:-1]
	return line

# states
# 1) looking for an entry (flagged by ':0 Hf')
# 2) collecting regexps

LOOKING = 0
COLLECTING = 1

def scan_tagfile (file):
	list = []
	state = LOOKING
	while 1:
		line = file.readline()
		if not line:
			break

		line = strip_eol (line)
		if not line or line[0] == '#':
			continue
		
		if state == LOOKING:
			if line == ':0 Hf':
				state = COLLECTING
				regs = []

		elif state == COLLECTING:
			if line[0] == '*':
				regs.append (line[2:])
			else:
				list.append (regs)
				state = LOOKING
	saved_syntax = regex.set_syntax (regex_syntax.RE_SYNTAX_AWK)
	try:
		list = map (compile_filter, list)
	finally:
		ignore = regex.set_syntax (saved_syntax)
	
	return list

def compile_filter (filter):
	result = []
	for r in filter:
		try:
			result.append (regex.compile (r))
		except regex.error, why:
			print 'error with regex <%s>' % repr(r)
			raise regex.error, why
	return result

if __name__ == '__main__':
	import sys
	print scan_tagfile (open (sys.argv[1]))



# -*- Mode: Python; tab-width: 4 -*-

# This file controls filtering.
# It is reloaded by the SPAM RELOAD command.

import regex_filter
import blacklist_filter
import multi_filter

def build_filter():
	import regex_filter
	import convert_procmail_tag

	f1 = regex_filter.regex_filter (
		convert_procmail_tag.scan_tagfile (
			open ('./filter_data/tag.txt', 'r')
			)
		)

	# To add in the blacklist filter, uncomment the
	# following lines:

	#f2 = blacklist_filter.blacklist_filter (
	#	'filter_data/filter.cf.data'
	#	)

	#return multi_filter.multi_filter (f1, f2)

	return f1



# -*- Mode: Python; tab-width: 4 -*-

RCS_ID =  '$Id: spamwall.py,v 1.4 1997/10/04 01:01:19 rushing Exp $'

# ======================================================================
# Copyright 1997 by Sam Rushing
# 
#                         All Rights Reserved
# 
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby
# granted, provided that the above copyright notice appear in all
# copies and that both that copyright notice and this permission
# notice appear in supporting documentation, and that the name of Sam
# Rushing not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission.
# 
# SAM RUSHING DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN
# NO EVENT SHALL SAM RUSHING BE LIABLE FOR ANY SPECIAL, INDIRECT OR
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
# OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
# NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
# CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
# ======================================================================

# techniques:
#
# 1) Access sendmail/smail using popen.
#  This is certainly possible, but less portable
#  and reliable: I've had trouble in the past trying to
#  do async i/o through popen.
#
# 2) Access SMTP on a different port.
#  The best way to set this up is to bind the server
#  to the loopback address (127.0.0.1).  This way only
#  the local machine can connect to the true SMTP server.
#  I use port 8025.

import asynchat
import asyncore
import socket
import string
import sys
import time
import ident

VERSION_STRING = string.split(RCS_ID)[2]

class smtp_proxy (asynchat.async_chat):
	def __init__ (self, channel):
		asynchat.async_chat.__init__ (self)
		self.channel = channel
		self.set_terminator ('\r\n')
		self.create_socket (socket.AF_INET, socket.SOCK_STREAM)
		self.connect (channel.server.proxy_addr)
		self.data = ''
		self.expect_data_reply = 0

	def handle_connect (self):
		pass

	def collect_incoming_data (self, data):
		self.data = self.data + data

	def found_terminator (self):
		line = self.data
		print '==>',repr(line)
		self.data = ''
		if self.expect_data_reply:
			self.expect_data_reply = 0
			if line[:3] != '354':
				# something's gone wrong on the server side, abort
				# this transaction with an error...
				self.channel.abort ("unexpected response from server: <%s>" % repr(line))
			else:
				# the client was already sent this reply by spamwall.
				pass
		else:
			if self.channel:
				self.channel.push (line+'\r\n')
		
	def handle_close (self):
		print 'proxy closed'
		self.channel.close()
		self.close()

# this is the server channel that redirects through the proxy

# The machine has 3 states:

COMMAND = 0
HEADER  = 1
BODY    = 2

# The connection is only 'hijacked' while in HEADER mode.
# everything else is passed through untouched.

class smtp_channel (asynchat.async_chat):
	
	ac_in_buffer_size = 8192

	def __init__ (self, server, conn, addr):
		asynchat.async_chat.__init__ (self, conn)
		self.server = server
		self.set_terminator ('\r\n')
		self.addr = addr
		self.proxy = smtp_proxy (self)
		self.line = ''
		self.headers = []
		self.state = COMMAND
		self.ident_username = None

	def collect_incoming_data (self, data):
		if self.state == BODY:
			# pass the body of the message directly
			self.proxy.push (data)
		else:
			self.line = self.line + data

	def found_terminator (self):
		line, self.line = self.line, ''

		print '<==', repr(line)

		if self.state == COMMAND:

			if line:
				command = string.lower (string.split(line)[0])
			else:
				command = ''

			if command == 'data':
				self.state = HEADER
				# don't send the DATA command to the real server yet, but go ahead
				# and send a response to the client.
				self.push ('354 Enter mail, end with "." on a line by itself\r\n')
			elif command == 'spam':
				# Intercept SPAM commands.
				sub_command = string.lower (string.split (line)[1])
				if sub_command == 'reload':
					import config
					reload (config)
					self.filter = config.build_filter()
					self.push ('filters reloaded successfully\r\n')
				elif sub_command == 'count':
					self.push ('%d messages rejected\r\n' %  self.server.rejections)
				elif sub_command == 'help':
					self.push ('SPAM count|help|reload|report\r\n')
				elif sub_command == 'report':
					f = self.server.filter
					if hasattr (f, 'report'):
						f.report(self)
					else:
						self.push ('nothing to report')
				else:
					self.push ('500 Unrecognized SPAM command\r\n')
			else:
				self.proxy.push (line+'\r\n')
		elif self.state == HEADER:
			if line == '':
				if self.feed_headers ():
					return
				# end of headers.
				# feed the headers to the real server.
				self.set_terminator ('\r\n.\r\n')
				self.state = BODY
			elif line == '.':
				# no body
				self.state = COMMAND
			else:
				self.headers.append (line)
		elif self.state == BODY:
			self.proxy.push ('\r\n.\r\n')
			self.state = COMMAND
			self.set_terminator ('\r\n')
		else:
			self.abort ("spamwall is confused")

	def feed_headers (self):
		# ok, we have the headers now.
		# check against filters and feed the data to the real server.

		if self.server.filter.scan (self.headers):

			# tell the server to forget it
			self.proxy.channel = None
			self.proxy.push ('QUIT\r\n')
			self.proxy.close_when_done()
			# say goodbye to the client
			self.push ('551 SPAM filter triggered.\r\n')
			self.close_when_done()

			if self.server.log_file:
				self.server.log_file.write (
					string.join (self.headers, '\n') + '\n\n'
					)
				self.server.log_file.flush()

			self.server.rejections = self.server.rejections + 1

			return 1

		# we survived the filter, now go on delivering the mail.

		if self.ident_username:
			user_ident = self.ident_username
		else:
			user_ident = 'no info'

		self.headers.insert (
			0,
			'X-SpamWall: Received via %s:%d (ident:%s)' % (
				self.addr[0],
				self.addr[1],
				user_ident
				)
			)

		self.proxy.push (
			'DATA\r\n' + string.join (self.headers, '\r\n') + '\r\n\r\n'
			)

		# tell the proxy class to ignore the real server's reply to DATA
		self.proxy.expect_data_reply = 1
		self.headers = []
		return 0

	def abort (self, message):
		self.proxy.push ('500 spamwall error (%s)\r\n' % message)
		self.proxy.close_when_done()
		self.close()

	def ident_callback (self, username):
		self.ident_username = username

	def handle_close (self):
		self.close()
		self.proxy.close()

class smtp_proxy_server (asyncore.dispatcher):

	channel_class = smtp_channel

	def __init__ (self, local_addr, proxy_addr, filter, do_ident=1, log_file=None):
		self.local_addr = local_addr
		self.proxy_addr = proxy_addr
		self.do_ident = do_ident
		self.log_file = log_file
		self.rejections = 0
		self.filter = filter
		asyncore.dispatcher.__init__ (self)
		self.create_socket (socket.AF_INET, socket.SOCK_STREAM)

		# try to re-use a server port if possible
		self.socket.setsockopt (
			socket.SOL_SOCKET, socket.SO_REUSEADDR,
			self.socket.getsockopt (socket.SOL_SOCKET, socket.SO_REUSEADDR) | 1
			)

		self.bind (local_addr)
		self.listen (5)
		print 'Proxy SMTP server started at %s\n\tLocal Address:%s\n\tRemote Address:%s\n' % (
			time.ctime(time.time()),
			local_addr,
			proxy_addr
			)
		self.proxy_addr = proxy_addr

	def handle_accept (self):
		conn, addr = self.accept()
		print 'incoming connection from %s' % repr(addr)
		sys.stdout.flush()
		c = self.channel_class (self, conn, addr)
		if self.do_ident:
			ident.ident_client (
				(addr[1], self.local_addr[1]), addr[0], c.ident_callback
				)

import os
if os.name == 'posix':
	def become_nobody ():
		import os
		import pwd
		nobody_uid = pwd.getpwnam ('nobody')[2]
		os.setuid (nobody_uid)

def run (local_addr, server_addr):
	log = open ('spamlog.txt', 'a')
	import config
	filter = config.build_filter()
	ps = smtp_proxy_server (local_addr, server_addr, filter, log_file=log)

	# os-specific:
	# become_nobody()

	asyncore.loop()

if __name__ == '__main__':
	run (('', 25), ('127.0.0.1', 8025))
